"""Tests if two images are the same on a pixel-by-pixel basis.

    Typical useage example:
    $ python image_diff.py image_a image_b
"""


#!/usr/bin/env python
# import os
import sys
import numpy as np
# import imageio
from PIL import Image
# from PIL import ImageChops


def same(file_a, file_b, verbose=0):
    """ Determines if two images are the same or different.

    Creates to image files from the image string arguments, then gets the
    pixel data from each image.  Subtracts pixel information of file_b from
    file_a, and creates a numpy array, which is the pixel-by-pixel difference
    of the two images.

    If the L2 norm of the difference array is less than a small tolerance, the
    two images are classified as the same.  Otherwise, they are classified as
    different.

    Args:
        file_a (string) The first of two image file names.
        file_b (string): The second of two image file names.
        verbose: (Boolean): True provides more command line feedback that False.

    Returns:
        Boolean:  True for images are the same; False for images are different.
    """

    same_xy_dimension = False
    same_mode = False
    same_channels = False
    same_pixels = False

    if verbose:
        print(f'Image 1 file name: {file_a}')
        print(f'Image 2 file name: {file_b}')

    try:
        with Image.open(file_a) as im_a, Image.open(file_b) as im_b:
            # im_a = Image.open(file_a)
            # im_b = Image.open(file_b)
            im_a_size = im_a.size
            im_b_size = im_b.size

            same_xy_dimension = im_a_size == im_b_size

            if verbose:
                print('Test 1 of 2: Dimensionality comparison')
                print(f'  (x,y) dimension of image 1 is {im_a_size}')
                print(f'  (x,y) dimension of image 2 is {im_b_size}')

            same_mode = im_a.mode == im_b.mode

            if same_xy_dimension and same_mode:

                data_a = np.array(im_a.getdata()).flatten()
                data_b = np.array(im_b.getdata()).flatten()

                same_channels = data_a.size == data_b.size

                if same_channels:
                    data_diff = data_a - data_b

                    data_diff_norm = np.linalg.norm(data_diff)
                    image_tol = 10.0  # tolerance for the L2norm to be same or different

                    # if np.abs(data_diff_norm) < image_tol:
                    #     same_pixels = True

                    same_pixels = np.abs(data_diff_norm) < image_tol

                    if verbose:
                        print(f'  Images (x,y) dimensions are the same? [T/F]: {same_xy_dimension}')
                        print(f'  Images have same mode? [T/F]: {same_mode}')
                        print(f'  Images have same number of channels? [T/F]: {same_channels}')
                        print('Test 2 of 2: Pixel-by-pixel comparison')
                        print(f'  Size of data image 1 is {data_a.size}')
                        print(f'  Size of data image 2 is {data_b.size}')
                        print(f'  Pixels are the same? [T/F]: {same_pixels}')

            else:
                if verbose:
                    print(f'  Images (x,y) dimensions are the same? [T/F]: {same_xy_dimension}')
                    print(f'  Images have same mode? [T/F]: {same_mode}')
                    print(f'  Images have same number of channels? [T/F]: {same_channels}')
                    print('Images are different.')

            if verbose:
                print(f'Returning {same_pixels}')

            return same_pixels

    except IOError:
        print('Error: could not open files:')
        print(f'  {file_a}')
        print(f'  {file_b}')


def main(argv):
    """ The main entry point for command line interation.

    Reads two image file string from the command line arguments and forward
    the file name strings to the worker method called 'same()'.

    Args:
        file_a: The first of two image file names.
        file_b: The second of two image file names.

    Returns:
        None
    """

    try:
        file_a = argv[0]
        file_b = argv[1]
    except IndexError as error:
        print(f'Error: {error}.')
        print('Abnormal script termination.')
        sys.exit('Two image files must be specified as arguments.')

    same(file_a, file_b, verbose=True)


if __name__ == '__main__':
    main(sys.argv[1:])



import os
import sys

# from importlib import import_module
# import importlib.util as ilu

import numpy as np
from scipy import signal
from scipy.integrate import cumtrapz

# from xybase import XYBase
from xyfigure.xybase import XYBase


class XYModel(XYBase):
    """The data to be plotted in XY format."""
    def __init__(self, guid, **kwargs):
        super().__init__(guid, **kwargs)

        self._skip_rows = kwargs.get('skip_rows', 0)
        self._skip_rows_footer = kwargs.get('skip_rows_footer', 0)
        self._xcolumn = kwargs.get('xcolumn', 0)  # default to the first column
        self._ycolumn = kwargs.get('ycolumn', 1)  # default to the second column

        rel_path_and_file = os.path.join(self._folder, self._file)  # relative to current run location
        self._data = np.genfromtxt(rel_path_and_file, dtype='float', delimiter=',', skip_header=self._skip_rows, skip_footer=self._skip_rows_footer, usecols=(self._xcolumn, self._ycolumn))

        # default value if plot_kwargs not client-supplied
        default = {'linewidth': 2.0, 'linestyle': '-'}
        self._plot_kwargs = kwargs.get('plot_kwargs', default)

        self._xscale = kwargs.get('xscale', 1.0)
        self._yscale = kwargs.get('yscale', 1.0)
        self._xoffset = kwargs.get('xoffset', 0.0)
        self._yoffset = kwargs.get('yoffset', 0.0)

        self._signal_process = kwargs.get('signal_process', None)

        if self._signal_process:
            for process_id in self._signal_process:
                process_dict = self._signal_process.get(process_id)
                key = next(iter(process_dict))
                value = process_dict[key]

                try:
                    method = getattr(self, key)
                    method(value)
                except AttributeError as error:
                    print(f'Error: invalid signal process key: {key}')
                    print(error.__class__.__name__)

                # process_module = module(f'{key}.process')
                # if process_module:
                #     process_object = getattr(process_module, 'Process')
                #     pobject = process_object(self._data, **value)
                #     # self._data[:, 1] = yfiltered  # overwrite y-axis now that it is filtered
                #     yfiltered = pobject.processed_data()
                #     self._data[:, 1] = yfiltered  # overwrite y-axis now that it is filtered

                # if key == 'butterworth':

                #     # process_module = import_module(f'{key}.process')
                #     # process_object = getattr(process_module, 'Process')


                #     fc = value.get('cutoff', None)  # Hz, cutoff frequency
                #     if fc is None:
                #         print('Error: keyword "cutoff" not found.')
                #         sys.exit('Abnormal termination.')

                #     filter_order = value.get('order', None)
                #     if filter_order is None:
                #         print('Error: keyword "order" not found.')
                #         sys.exit('Abnormal termination.')

                #     filter_type = value.get('type', None)
                #     if filter_type is None:
                #         print('Error: keyword "type" not found.')
                #         sys.exit('Abnormal termination.')

                #     print('Signal process ' + key + ' with:')
                #     print(f'  cutoff frequency = {fc} Hz')
                #     print(f'  filter order = {filter_order}')
                #     print('  filter type = ' + filter_type)

                #     dt = self._data[1, 0] - self._data[0, 0]  # sample delta t
                #     print(f'  sample delta t = {dt} seconds')

                #     fs = 1.0/dt  # Hz
                #     print(f'  sample frequency = {fs} Hz')

                #     Wn = fc / (fs / 2)  # normalized critical frequency
                #     print(f'  normalized critical frequency = {Wn} Hz/Hz')

                #     b, a = signal.butter(filter_order, Wn, filter_type)
                #     yfiltered = signal.filtfilt(b, a, self._data[:, 1])
                #     self._data[:, 1] = yfiltered  # overwrite

                #     serialize = value.get('serialize', 0)  # default is not to serialize
                #     if serialize:
                #         folder = value.get('folder', '.')  # default to current folder
                #         filename = value.get('file', str(process_id) + '.csv')

                #         #  abs_path = absolute_path(folder)
                #         #  # defaults to the process id
                #         #  abs_path_and_file = os.path.join(abs_path, filename)
                #         #  np.savetxt(abs_path_and_file, np.transpose([self._data[:, 0], self._data[:, 1]]), delimiter=',')
                #         #  print(f'  serialized file = {filename}')
                #         self.serialize(folder, filename)

                # elif key == 'gradient':

                #     gradient_order = value.get('order', None)
                #     if gradient_order is None:
                #         print('Error: keyword "order" not found.')
                #         sys.exit('Abnormal termination.')

                #     print('Signal process: ' + key + ' applied ' + str(gradient_order) + ' time(s)')
                #     # numerical gradient
                #     # https://docs.scipy.org/doc/numpy/reference/generated/numpy.gradient.html
                #     # for k in range(value):
                #     for k in range(gradient_order):
                #         ydot = np.gradient(self._data[:, 1], self._data[:, 0], edge_order=2)
                #         self._data[:, 1] = ydot  # overwrite
                #         print(f'  Derivative {k+1} completed.')

                #     serialize = value.get('serialize', 0)  # default is not to serialize
                #     if serialize:
                #         folder = value.get('folder', '.')  # default to current folder
                #         filename = value.get('file', str(process_id) + '.csv')

                #         # abs_path = absolute_path(folder)
                #         # # defaults to the process id
                #         # abs_path_and_file = os.path.join(abs_path, filename)
                #         # np.savetxt(abs_path_and_file, np.transpose([self._data[:, 0], self._data[:, 1]]), delimiter=',')
                #         # print(f'  serialized file = {filename}')
                #         self.serialize(folder, filename)

                # elif key == 'integration':

                #     integration_order = value.get('order', None)
                #     if integration_order is None:
                #         print('Error: keyword "order" not found.')
                #         sys.exit('Abnormal termination.')

                #     print('Signal process: ' + key + ' applied ' + str(integration_order) + ' time(s) with')

                #     default_ics = np.zeros(integration_order)
                #     ics = value.get('initial_conditions', default_ics)

                #     print(f'initial condition(s) as {ics}')

                #     if len(ics) != integration_order:
                #         print('Error: length of initial condition(s) array')
                #         print('must be equal to the order of integration.')
                #         print(f'Specified order = {integration_order}')
                #         print(f'Length of intial condition(s) array = {len(ics)}')
                #         sys.exit('Abnormal termination.')

                #     # https://docs.scipy.org/doc/numpy/reference/generated/numpy.trapz.html
                #     # https://docs.scipy.org/doc/scipy-0.14.0/reference/generated/scipy.integrate.cumtrapz.html
                #     for k in range(integration_order):
                #         # inty = np.trapz(self._data[:, 1], self._data[:, 0]) + ics[k]
                #         # inty = cumtrapz(self._data[:, 1], self._data[:, 0], initial=ics[k])
                #         inty = cumtrapz(self._data[:, 1], self._data[:, 0], initial=0) + ics[k]
                #         self._data[:, 1] = inty  # overwrite
                #         print(f'  Integral {k+1} completed.')

                #     serialize = value.get('serialize', 0)  # default is not to serialize
                #     if serialize:
                #         folder = value.get('folder', '.')  # default to current folder
                #         filename = value.get('file', str(process_id) + '.csv')

                #         self.serialize(folder, filename)

                # else:
                #     print('Error: Signal process key not implemented.')
                #     sys.exit('Abnormal termination.')

                print('  Signal process "' + key + '" completed.')

    @property
    def x(self):
        """Returns the model's x data."""
        return self._data[:, 0] * self._xscale + self._xoffset

    @property
    def y(self):
        """Returns the model's y data."""
        return self._data[:, 1] * self._yscale + self._yoffset

    @property
    def plot_kwargs(self):
        """Returns kwargs passed to matplotlib.pyplot.plot()."""
        return self._plot_kwargs

    def serialize(self, folder, filename):  # extend base class
        super().serialize(folder, filename)
        np.savetxt(self._path_file_output, np.transpose([self._data[:, 0], self._data[:, 1]]), delimiter=',')
        print(f'  Serialized model to: {self._path_file_output}')


    def butterworth(self, value):

        fc = value.get('cutoff', None)  # Hz, cutoff frequency
        if fc is None:
            print('Error: keyword "cutoff" not found.')
            sys.exit('Abnormal termination.')
        
        filter_order = value.get('order', None)
        if filter_order is None:
            print('Error: keyword "order" not found.')
            sys.exit('Abnormal termination.')
        
        filter_type = value.get('type', None)
        if filter_type is None:
            print('Error: keyword "type" not found.')
            sys.exit('Abnormal termination.')
        
        print('Signal process "butterworth" with:')
        print(f'  cutoff frequency = {fc} Hz')
        print(f'  filter order = {filter_order}')
        print('  filter type = ' + filter_type)
        
        dt = self._data[1, 0] - self._data[0, 0]  # sample delta t
        print(f'  sample delta t = {dt} seconds')
        
        fs = 1.0/dt  # Hz
        print(f'  sample frequency = {fs} Hz')
        
        Wn = fc / (fs / 2)  # normalized critical frequency
        print(f'  normalized critical frequency = {Wn} Hz/Hz')
        
        b, a = signal.butter(filter_order, Wn, filter_type)
        yfiltered = signal.filtfilt(b, a, self._data[:, 1])
        self._data[:, 1] = yfiltered  # overwrite
        
        serialize = value.get('serialize', 0)  # default is not to serialize
        if serialize:
            folder = value.get('folder', '.')  # default to current folder
            file_output = value.get('file', None)

            if file_output is None:
                print('Error: keyword "file" not found.')
                sys.exit('Abnormal termination.')
            else:
                self.serialize(folder, file_output)


    def gradient(self, value):

        gradient_order = value.get('order', None)
        if gradient_order is None:
            print('Error: keyword "order" not found.')
            sys.exit('Abnormal termination.')

        print('Signal process: "gradient" applied ' + str(gradient_order) + ' time(s)')
        # numerical gradient
        # https://docs.scipy.org/doc/numpy/reference/generated/numpy.gradient.html
        # for k in range(value):
        for k in range(gradient_order):
            ydot = np.gradient(self._data[:, 1], self._data[:, 0], edge_order=2)
            self._data[:, 1] = ydot  # overwrite
            print(f'  Derivative {k+1} completed.')

        serialize = value.get('serialize', 0)  # default is not to serialize
        if serialize:
            folder = value.get('folder', '.')  # default to current folder
            file_output = value.get('file', None)

            if file_output is None:
                print('Error: keyword "file" not found.')
                sys.exit('Abnormal termination.')
            else:
                self.serialize(folder, file_output)

    def integration(self, value):

        integration_order = value.get('order', None)
        if integration_order is None:
            print('Error: keyword "order" not found.')
            sys.exit('Abnormal termination.')

        print('Signal process: "integration" applied ' + str(integration_order) + ' time(s) with')

        default_ics = np.zeros(integration_order)
        ics = value.get('initial_conditions', default_ics)

        print(f'initial condition(s) as {ics}')

        if len(ics) != integration_order:
            print('Error: length of initial condition(s) array')
            print('must be equal to the order of integration.')
            print(f'Specified order = {integration_order}')
            print(f'Length of intial condition(s) array = {len(ics)}')
            sys.exit('Abnormal termination.')

        # https://docs.scipy.org/doc/numpy/reference/generated/numpy.trapz.html
        # https://docs.scipy.org/doc/scipy-0.14.0/reference/generated/scipy.integrate.cumtrapz.html
        for k in range(integration_order):
            # inty = np.trapz(self._data[:, 1], self._data[:, 0]) + ics[k]
            # inty = cumtrapz(self._data[:, 1], self._data[:, 0], initial=ics[k])
            inty = cumtrapz(self._data[:, 1], self._data[:, 0], initial=0) + ics[k]
            self._data[:, 1] = inty  # overwrite
            print(f'  Integral {k+1} completed.')

        serialize = value.get('serialize', 0)  # default is not to serialize
        if serialize:
            folder = value.get('folder', '.')  # default to current folder
            file_output = value.get('file', None)

            if file_output is None:
                print('Error: keyword "file" not found.')
                sys.exit('Abnormal termination.')
            else:
                self.serialize(folder, file_output)



import os
from datetime import datetime

import matplotlib.pyplot as plt
from PIL import Image

# from xybase import XYBase
from xyfigure.xybase import XYBase

#from matplotlib import rc
#rc('font', **{'family': 'serif', 'serif': ['Computer Modern Roman']})
#rc('text', usetex=True)

class XYView(XYBase):
    """Creates a view that sees models."""
    def __init__(self, guid, **kwargs):
        super().__init__(guid, **kwargs)
        self._models = []
        self._model_keys = kwargs.get('model_keys', None)
        self._figure = None
        # self._folder = kwargs.get('folder', None)
        self._file_base = self._file.split('.')[0]

        # default value if figure_kwargs not client-supplied
        self._title = kwargs.get('title', 'default title')
        self._xlabel = kwargs.get('xlabel', 'default x axis label')
        self._ylabel = kwargs.get('ylabel', 'default y axis label')

        self._xticks = kwargs.get('xticks', None)
        self._yticks = kwargs.get('yticks', None)

        self._x_log_scale = kwargs.get('x_log_scale', None)

        # default = {'scale': 1, 'label': 'ylabel_rhs', 'verification': 0}
        self._yaxis_rhs = kwargs.get('yaxis_rhs', None)

        self._size = kwargs.get('size', [11.0, 8.5])  # inches, U.S. paper, landscape
        self._dpi = kwargs.get('dpi', 300)
        self._xlim = kwargs.get('xlim', None)
        self._ylim = kwargs.get('ylim', None)

        self._background_image = kwargs.get('background_image', None)

        self._display = kwargs.get('display', True)
        self._details = kwargs.get('details', True)
        # self._serialize = kwargs.get('serialize', False) # moved up to XYBase
        self._latex = kwargs.get('latex', False)
        if self._latex:
            from matplotlib import rc
            #rc('font', **{'family': 'serif', 'serif': ['Computer Modern Roman']})
            rc('text', usetex=True)
            rc('font', family='serif')

    @property
    def models(self):
        return self._models

    @models.setter
    def models(self, value):
        self._models = value

    @property
    def model_keys(self):
        return self._model_keys

    def figure(self):
        """Create a figure (view) of the registered models to the screen."""
        if self._figure is None:

            # dpi versus fig size
            # https://stackoverflow.com/questions/47633546/relationship-between-dpi-and-figure-size
            # fig, ax = plt.subplots(nrows=1, dpi=self._dpi)
            self._figure, ax = plt.subplots(nrows=1, dpi=self._dpi)
            print(f'  Figure dpi set to {self._dpi}')

            # ax.ticklabel_format(axis='y', style='scientific')
            # ax.ticklabel_format(axis='both', style='scientific', scilimits=(0,0))

            # https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.figure.Figure.html#matplotlib.figure.Figure.set_size_inches
            # fig.set_size_inches(self._size)
            self._figure.set_size_inches(self._size)
            print('  Figure size set to ' +  str(self._size) + ' inches.')

            if self._background_image:
                folder = self._background_image.get('folder', '.')
                file = self._background_image.get('file', None)
                rel_path_and_file = os.path.join(folder, file)  # relative to current run location
                im = Image.open(rel_path_and_file)

                left = self._background_image.get('left', 0.0)
                right = self._background_image.get('right', 1.0)
                bottom = self._background_image.get('bottom', 0.0)
                top = self._background_image.get('top', 1.0)
                al = self._background_image.get('alpha', 1.0)

                # https://github.com/matplotlib/matplotlib/issues/3173
                # https://matplotlib.org/3.1.1/tutorials/intermediate/imshow_extent.html
                bounds = [left, right, bottom ,top]
                im = ax.imshow(im, zorder=0, extent=bounds, alpha=al, aspect='auto')

            for model in self._models:
                # needs rearchitecting, a logview descends from a view
                if self._x_log_scale:  # needs rearchitecting
                    ax.semilogx(model.x, model.y, **model.plot_kwargs)
                else:
                    ax.plot(model.x, model.y, **model.plot_kwargs)

            if self._xticks:
                ax.set_xticks(self._xticks)

            if self._yticks:
                ax.set_yticks(self._yticks)

            if self._xlim:
                ax.set_xlim(self._xlim)

            if self._ylim:
                ax.set_ylim(self._ylim)

            if self._yaxis_rhs:
                rhs_axis_scale = self._yaxis_rhs.get('scale', 1)
                rhs_axis_label = self._yaxis_rhs.get('label', None)
                # rhs_yticks_str = self._yaxis_rhs.get('yticks', None)
                rhs_yticks = self._yaxis_rhs.get('yticks', None)

                # ax2 = fig.add_subplot(111, sharex=ax, frameon=False)
                ax2 = self._figure.add_subplot(111, sharex=ax, frameon=False)
                bottom, top = ax.get_ylim()  # get from left-hand-side y-axis
                ax2.set_ylim(rhs_axis_scale * bottom, rhs_axis_scale * top)
                ax2.yaxis.tick_right()
                # ax2.ticklabel_format(axis='both', style='scientific', scilimits=(0,0))
                # ax.ticklabel_format(axis='both', style='scientific', scilimits=(0,0))
                # _ticklabel_format = self._yaxis_rhs.get('ticklabel_format', None)
                # _ticklabel_format = self._yaxis_rhs.get('ticklabel_format', None)
                # if _ticklabel_format:
                #     scilimits_str = _ticklabel_format.get('scilimitsl', "(0, 0)")
                #     ax2.ticklabel_format(**_ticklabel_format)
                # ax2.ticklabel_format(axis='both', style='scientific', scilimits=(0,0))
                # plt.ticklabel_format(axis='y', style='scientific', useOffset=False)
                # ax2.ticklabel_format(axis='y', style='scientific', useOffset=False)
                if rhs_yticks:
                    ax2.set_yticks(rhs_yticks)
                ax2.yaxis.set_label_position('right')
                ax2.set_ylabel(rhs_axis_label)

            # fig.suptitle(self._title)
            self._figure.suptitle(self._title)
            ax.set_xlabel(self._xlabel)
            ax.set_ylabel(self._ylabel)
            ax.grid()
            ax.legend()

            if self._details:
                now = datetime.now()
                now_str = now.strftime("%Y-%m-%d %H:%M:%S")
                user = str(os.getlogin())
                host = str(os.getenv('HOSTNAME'))
                details_str = self._file + ' created ' + now_str + ' by ' + user + ' on ' + host
                ax.set_title(details_str, fontsize=10, ha='center', color='dimgray')

            if self._display:
                plt.show()

            if self._serialize:
                self.serialize(self._folder, self._file)

            plt.close('all')
            self._figure = None

    def serialize(self, folder, filename):  # extend base class
        super().serialize(folder, filename)
        self._figure.savefig(self._path_file_output, dpi=self._dpi, bbox_inches='tight')  # avoid cutoff of labels
        print(f'  Serialized view to: {self._path_file_output}')



"""
This module is a unit test of the image_diff.py script.
To run

# python image_diff_test.py                 # or
$ python -m unittest image_diff_test.py     # for default interaction, and
$ python -m unittest -v image_diff_test.py  # for higher verbosity

"""
# import unittest
from unittest import TestCase, main
from image_diff import same

class TestImageDiff(TestCase):

    @classmethod
    def setUpClass(cls):
        cls._path_str = 'sibl/xyfigure/test/image_diff_test.py'
        print(cls._path_str + ' initialized.')

        cls._orig = 'image_diff_test.png'
        cls._same = 'image_diff_test_clone.png'
        cls._diff = 'image_diff_test_diff.png'

    @classmethod
    def tearDownClass(cls):
        print(cls._path_str + ' completed.')

    def test_same(self):
        # self.assertTrue(same(self._orig, self._same, verbose=True))
        self.assertTrue(same(self._orig, self._same))

    def test_different(self):
        self.assertFalse(same(self._orig, self._diff))
        self.assertFalse(same(self._same, self._diff))


if __name__ == '__main__':
    main()  # calls unittest.main()



import numpy as np
import matplotlib.pyplot as plt
from scipy import signal

# references
# https://dsp.stackexchange.com/questions/49460/apply-low-pass-butterworth-filter-in-python

# client filter input begin
fs = 100  # Hz, sampling frequency in t_v_sines.csv
fc = 10  # cutoff frequency of the filter
filter_order = 4  
filter_type = 'low'
# client filter input end

# client signal data begin
t = np.arange(fs) / fs  # seconds, [0, 1) by 0.01 seconds
t2 = np.arange(2 * fs) / fs  # seconds, [0, 2) by 0.01 seconds
# print('time vector: ')
# print(t)
# print('time 2 vector: ')
# print(t2)
f1 = 2.0  # Hz, frequency of u1
f2 = 20.0 # Hz, frequency of u2

# three signals
u1 = np.sin(2 * np.pi * f1 * t2)
u2 = np.sin(2 * np.pi * f2 * t2)
u3 = u1 + u2
# client signal data end


# Butterworth filter service begin
Wn = fc / (fs / 2)  # normalized critical frequency
b, a = signal.butter(filter_order, Wn, filter_type)
# https://docs.scipy.org/doc/scipy/reference/generated/scipy.signal.butter.html
u3_filtered = signal.filtfilt(b, a, u3)

# visualization service begin
u1str = 'u1 = sin(' + str(f1) + ' * 2 pi t)'
u2str = 'u2 = sin(' + str(f2) + ' * 2 pi t)'
plt.plot(t2, u1, label=u1str, linewidth=3, alpha=0.7)
plt.plot(t2, u2, label=u2str)
plt.plot(t2, u3, label='u3 = u1 + u2')
plt.plot(t2, u3_filtered, label='u3 filtered', linestyle='dashed')
plt.legend()
plt.show()




#!/usr/bin/env python
# Client to create figures for Military Specification journal manuscript.
# To run from command line with Python3:
# [base_directory]: $ python ~/sibl/xyfigure/client.py input_file.json

# from xymodel import XYModel
from xyfigure.xymodel import XYModel
# from xyview import XYView
from  xyfigure.xyview import XYView

## Figure Factory
FACTORY_ITEMS = {
    'model': XYModel,
    'view': XYView
}

class XYFactory:
    """The one and only (singleton) factory for XY items."""
    @staticmethod
    def create(item, **kwargs):
        "Main factory method, returns XY objects."
        instance = FACTORY_ITEMS.get(kwargs['class'], None)
        if instance:
            return instance(item, **kwargs)

        # If we get here, we did not return an instance, so warn.
        print(f'Warning: {item} requested but not provided by this factory, returning None.')
        return None



#!/usr/bin/env python
# import os
import sys
import json
# from factory import XYFactory
from xyfigure.factory import XYFactory
# from xymodel import XYModel
from xyfigure.xymodel import XYModel
# from xyview import XYView
from xyfigure.xyview import XYView


def main(argv):
    """Client to generate a XYFigure from an 'input_file.json' file.

    Preconditions:
    $ module load anaconda3
    ~/sibl/io/input_file.json  # database of XYFigure objects to create

    Use:
    $ cd ~/sibl/io/<path_containing_input_file.json>/
    $ python ~/sibl/xyfigure/client.py input_file.json

    Example Input:
    $ cd ~/sibl/io/mil_spec_paper/
    $ python ~/sibl/xyfigure/client.py MHSRS_exp_only.json

    Example Output:
    ~/sibl/io/mil_spec_paper/fig/MHSRS_exp_only.pdf  # output figure

    """

    help_string = '$ python ~/sibl/xyfigure/client.py input_file.json'
    try:
        input_file = argv[0]
    except IndexError as error:
        print(f'Error: {error}.')
        print('check script pattern: ' + help_string)
        print('Abnormal script termination.')
        sys.exit('No input file specified.')

    with open(input_file) as f:
        database = json.load(f)

    items = []  # cart of items is empty, fill from factory
    # factory = XYFactory()  # it's static!

    for item in database:
        kwargs = database[item]
        i = XYFactory.create(item, **kwargs)
        if i:
            items.append(i)
        else:
            print('Item is None from factory, nothing added to Client items.')

    models = [i for i in items if isinstance(i, XYModel)]
    views = [i for i in items if isinstance(i, XYView)]

    for view in views:
        print(f'Creating view with guid = "{view.guid}"')
        if view.model_keys: # register only selected models with current view
            print(f'  Adding {view.model_keys} model(s) to current view.')
            view.models = [m for m in models if m.guid in view.model_keys]
            view.figure()  # must be within this subset scope
        else: 
            print('  Adding all models to current view.')
            view.models = models  # register all models with current view
            view.figure()  # must be within this subset scope

    print('=======================')
    print('End of client execution.')

if __name__ == '__main__':
    main(sys.argv[1:])



"""
This module is a unit test of the xyfigure service.
To run

$ python xyfigure_test.py                      # for terse interaction,
$ python -m unittest xyfigure_test             # for default interaction,
$ python -m unittest -v xyfigure_test          # for higher verbosity, and
to test just one of the test methods:

$ python -m unittest xyfigure_test.TestImageDiff.test_same  # e.g., to test the test_same() method

"""
# standard library imports
import json
import os
import sys
# import unittest
from unittest import TestCase, main

# third-party imports

# local imports
# from test.image_diff import same
# import test.image_diff as idff
# import image_diff as idff
# import client as xyfigure_client
# from image_diff import same
from xyfigure.test.image_diff import same

# from ..client import main
#from .. client import main
# sys.path.insert(0, '../')
# import xyfigure as xyfigure_client  # from parent directory
# import client as xyfigure_client  # from parent directory
import xyfigure.client as xyfigure_client 


class TestImageDiff(TestCase):

    # https://stackoverflow.com/questions/23667610/what-is-the-difference-between-setup-and-setupclass-in-python-unittest/23670844
    # The difference manifests itself when you have more than one test method
    # in your class. setUpClass and tearDownClass are run once for the whole class;
    # setUp and tearDown are run before and after each test method.

    @classmethod
    def setUpClass(cls):
        cls._path_str = 'sibl/xyfigure/test/xyfigure_test.py'
        print(cls._path_str + ' initialized.')

        cls._orig = 'image_diff_test.png'
        cls._clone = 'image_diff_test_clone.png'
        cls._diff = 'image_diff_test_diff.png'

        cls._rgb_orig = 'H_460_460_px_RGB_634800.png'
        cls._rgb_clone = 'H_460_460_px_RGB_634800_clone.png'

        cls._rgba_orig = 'H_460_460_px_RGBA_846400.png'
        cls._rgba_clone = 'H_460_460_px_RGBA_846400_clone.png'
        cls._rgba_diff = 'H_460_460_px_RGBA_846400_diff.png'

        cls._out = 'temp'  # put test files into sibl/xyfigure/test/temp/

    @classmethod
    def tearDownClass(cls):
        print(cls._path_str + ' completed.')

    @classmethod
    def compare_test_to_reference(cls, jfile):
        print(f'Reference json file is {jfile}')

        jfile_test = os.path.join(cls._out, jfile.split('.')[0] + '_test.json')
        print(f'Test json file is {jfile_test}')

        with open(jfile, 'r') as fin:
            dict_test = json.load(fin)  # make a copy of reference json
            # print(dict_test)  # for debug

            file_a = dict_test['figure']['file']
            print(f'Reference png file is {file_a}')

            file_b = os.path.join(cls._out, file_a.split('.')[0] + '_test.png')
            print(f'Test png file is {file_b}')

            dict_test['figure']['file'] = file_b  # replace png file name
            # print(dict_test)  # for debug

            with open(jfile_test, 'w') as outfile:
                json.dump(dict_test, outfile)

            xyfigure_client.main([jfile_test])
        
            return (file_a, file_b)

    def test_same(self):
        # self.assertTrue(same(self._orig, self._same, verbose=True))
        self.assertTrue(same(self._orig, self._clone))
        self.assertTrue(same(self._rgb_orig, self._rgb_clone))
        self.assertTrue(same(self._rgba_orig, self._rgba_clone))

    def test_different(self):
        self.assertFalse(same(self._orig, self._diff))
        self.assertFalse(same(self._clone, self._diff))

        self.assertFalse(same(self._rgb_orig, self._rgba_orig))

        self.assertFalse(same(self._rgba_orig, self._rgba_diff))
        self.assertFalse(same(self._rgba_clone, self._rgba_diff))

    def test_cosines_prefilter(self):
        jfile = 'cosines-prefilter.json'
        fa, fb = self.compare_test_to_reference(jfile)
        self.assertTrue(same(fa, fb))

    def test_cosines_postfilter(self):
        jfile = 'cosines-postfilter.json'
        fa, fb = self.compare_test_to_reference(jfile)
        self.assertTrue(same(fa, fb))

    def test_integration(self):
        jfile = 'zeros-int3.json'
        fa, fb = self.compare_test_to_reference(jfile)
        self.assertTrue(same(fa, fb))

    def test_signal_process_serialize(self):
        jfile = 'signal_process_serialize.json'
        fa, fb = self.compare_test_to_reference(jfile)
        self.assertTrue(same(fa, fb))

    def test_sine(self):
        jfile = 't_v_sines.json'
        fa, fb = self.compare_test_to_reference(jfile)
        self.assertTrue(same(fa, fb))

    def test_sine_time_derivative(self):
        jfile = 't_v_sines_ddt1.json'
        fa, fb = self.compare_test_to_reference(jfile)
        self.assertTrue(same(fa, fb))

    def test_sines_prefilter(self):
        jfile = 't_v_sines_prefilter.json'
        fa, fb = self.compare_test_to_reference(jfile)
        self.assertTrue(same(fa, fb))

    def test_sines_postfilter(self):
        jfile = 't_v_sines_postfilter.json'
        fa, fb = self.compare_test_to_reference(jfile)
        self.assertTrue(same(fa, fb))

    def test_quadratic(self):
        jfile = 'u-squared.json'
        fa, fb = self.compare_test_to_reference(jfile)
        self.assertTrue(same(fa, fb))

    def test_quadratic_ddt1(self):
        jfile = 'u-squared-ddt1.json'
        fa, fb = self.compare_test_to_reference(jfile)
        self.assertTrue(same(fa, fb))

    def test_quadratic_ddt2(self):
        jfile = 'u-squared-ddt2.json'
        fa, fb = self.compare_test_to_reference(jfile)
        self.assertTrue(same(fa, fb))

    def test_quadratic_ddt3(self):
        jfile = 'u-squared-ddt3.json'
        fa, fb = self.compare_test_to_reference(jfile)
        self.assertTrue(same(fa, fb))


if __name__ == '__main__':
    main()  # calls unittest.main()
